
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;
using System.Text.RegularExpressions;

namespace ClarionDEMO
{
	class MainClass
	{
		#region properties

		private WorldServer worldServer = null;
		private ClarionAgent agent;
		double prad = 0;

		String creatureId = String.Empty;
		String creatureName = String.Empty;

		#endregion

		#region constructor

		//Dictionary<string, LeafletItem> Consolidatedleaflets;

		//Dictionary<string,int> leafletsTotal = new Dictionary<string,int>();
		//Dictionary<string,int> leafletsCollected = new Dictionary<string,int>();

		public MainClass ()
		{
			Console.WriteLine ("Clarion Demo V0.4");
			agent = new ClarionAgent ();
			agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler (agent_OnNewVisualSensorialInformationHandler);
			agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler (agent_OnNewExternalActionSelectedHandler);
			try {
				worldServer = new WorldServer ("localhost", 4011);
				String message = worldServer.Connect ();

				if (worldServer != null && worldServer.IsConnected) {
					Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
					//worldServer.SendWorldReset ();
					worldServer.NewCreature (400, 200, 0, out creatureId, out creatureName);
					worldServer.SendCreateLeaflet ();
					worldServer.NewBrick (4, 747, 2, 800, 567);
					worldServer.NewBrick (4, 50, -4, 747, 47);
					worldServer.NewBrick (4, 49, 562, 796, 599);
					worldServer.NewBrick (4, -2, 6, 50, 599);

					//foreach (var pair in consolidatedleaflets) {
					//	leafletsTotal.Add(pair.Key,pair.Value.totalNumber);
					//	leafletsCollected.Add(pair.Key, pair.Value.collected);
					//}

					if (!String.IsNullOrWhiteSpace (creatureId)) {
						worldServer.SendStartCamera (creatureId);
						worldServer.SendStartCreature (creatureId);
					}

					Console.Out.WriteLine ("Creature created with name: " + creatureId + "\n");
					agent.Run ();
					Console.Out.WriteLine ("Running Simulation ...\n");
				}
			} catch (WorldServerInvalidArgument invalidArtgument) {
				Console.Out.WriteLine (String.Format ("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
			} catch (WorldServerConnectionError serverError) {
				Console.Out.WriteLine (String.Format ("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
			} catch (Exception ex) {
				Console.Out.WriteLine (String.Format ("[ERROR] Unknown Error: {0}\n", ex.Message));
			}
		}

		#endregion

		#region Methods

		public static void Main (string[] args)
		{
			new MainClass ();
		}

		//Hirley-->>
		Dictionary<string, Thing> foodsDict = null;
		Dictionary<string, Thing> jewelsDict = null;
		Thing itemToGet = null;
		//<<--Hirley

		IList<Thing> agent_OnNewVisualSensorialInformationHandler ()
		{

			IList<Thing> response = null;

			//if (Consolidatedleaflets == null){
			//	Dictionary<string, LeafletItem> consolidatedleafletsTmp = agent.Consolidatedleaflets;	
			//	if (consolidatedleafletsTmp != null) {
			//		foreach (var pair in consolidatedleafletsTmp) {
			//			if (pair.Value.totalNumber > 0) {
			//				if (Consolidatedleaflets == null) {
			//					Consolidatedleaflets = new Dictionary<string,LeafletItem> ();
			//				}
			//				Consolidatedleaflets.Add (pair.Key, pair.Value);
			//			}
			//		}
			//	}
			//}
				

			foodsDict = new Dictionary<string, Thing>();
			jewelsDict = new Dictionary<string, Thing>();
				
			if (worldServer != null && worldServer.IsConnected) {
				response = worldServer.SendGetCreatureState (creatureName);
				prad = (Math.PI / 180) * response.First ().Pitch;
				while (prad > Math.PI)
					prad -= 2 * Math.PI;
				while (prad < -Math.PI)
					prad += 2 * Math.PI;

				//Hirley-->>
				foreach (Thing item in response) {
					switch (item.CategoryId) {
					case Thing.categoryPFOOD:
					case Thing.CATEGORY_FOOD:
					case Thing.CATEGORY_NPFOOD:
						if (!foodsDict.ContainsKey (item.Name)) {
							Console.WriteLine ("[" + creatureName + "] " + "I see food " + item.Name);
							foodsDict.Add (item.Name, item);
						}	
						break;
					case Thing.CATEGORY_JEWEL:
						if (!jewelsDict.ContainsKey (item.Name)) {
							//if (Consolidatedleaflets.ContainsKey(item.Material.Color)){
								//var leafletItem = Consolidatedleaflets [item.Material.Color];
								//if (leafletItem.totalNumber > leafletItem.collected) {
								if (agent.getTotalLeaflet(item.Material.Color) > agent.getCollectedLeaflet(item.Material.Color)) {
									Console.WriteLine ("[" + creatureName + "] " + "I see jewel " + item.Material.Color + " [" + agent.getCollectedLeaflet(item.Material.Color) + " of " + agent.getTotalLeaflet(item.Material.Color) + "]");
									jewelsDict.Add (item.Name, item);
								} else {
									Console.WriteLine ("[" + creatureName + "] " + "I see jewel " + item.Material.Color + " [" + agent.getCollectedLeaflet(item.Material.Color) + " of " +agent.getTotalLeaflet(item.Material.Color) + "] but don't want to collect it.");
								}
							//}
						}
						break;
					}
				}
				//<<--Hirley
			}

			return response;
		}

		void agent_OnNewExternalActionSelectedHandler (ClarionAgentActionType externalAction)
		{
			Thread.CurrentThread.CurrentCulture = new CultureInfo ("en-US");
			if (worldServer != null && worldServer.IsConnected) {
				switch (externalAction) {
				case ClarionAgentActionType.DO_NOTHING:
                        // Do nothing as the own value says
					break;
				case ClarionAgentActionType.ROTATE_CLOCKWISE:
					{
						double newprad = prad + Math.PI / 1000;
						while (newprad > Math.PI)
							newprad -= 2 * Math.PI;
						while (newprad < -Math.PI)
							newprad += 2 * Math.PI;
						worldServer.SendSetAngle (creatureId, 0, 0, newprad);
						break;
					}
				
				//Hirley-->>
				case ClarionAgentActionType.GO_AHEAD:
					{

						Dictionary<String,Thing> itemsDict = new Dictionary<String, Thing> ();
						
						if (foodsDict != null && foodsDict.Count > 0) {
							foreach (Thing food in foodsDict.Values) {
								itemsDict.Add (food.Name, food);
							}
						}
						if (jewelsDict != null && jewelsDict.Count > 0) {
							foreach (Thing jewel in jewelsDict.Values) {
								itemsDict.Add (jewel.Name, jewel);
							}
						}

						if (itemsDict.Count > 0) {
							foreach (Thing item in itemsDict.Values) {
								Console.WriteLine ("[" + creatureName + "] " + "Going to " + item.Material.Color);
								worldServer.SendSetGoTo (creatureId, 1, 1, item.comX, item.comY);
								itemToGet = item;
							}
						} else {
							double newprad = prad + Math.PI / 1000;
							while (newprad > Math.PI)
								newprad -= 2 * Math.PI;
							while (newprad < -Math.PI)
								newprad += 2 * Math.PI;
							worldServer.SendSetAngle (creatureId, 0, 0, newprad);
						}
						break;
					}
				case ClarionAgentActionType.GET_ITEM:
					if (itemToGet != null) {
						var worldServer2 = worldServer;
						switch (itemToGet.CategoryId) {
						case Thing.categoryPFOOD:
						case Thing.CATEGORY_FOOD:
						case Thing.CATEGORY_NPFOOD:
							Console.WriteLine ("[" + creatureName + "] " + "Eating " + itemToGet.Name);
							worldServer.SendEatIt (creatureId, itemToGet.Name);
							if (foodsDict.ContainsKey (itemToGet.Name)) {
								foodsDict.Remove (itemToGet.Name);
							}
							break;
						case Thing.CATEGORY_JEWEL:
							if (jewelsDict.ContainsKey (itemToGet.Name)) {
								//if (Consolidatedleaflets.ContainsKey (itemToGet.Material.Color)) {
									//var leafletItem = Consolidatedleaflets [itemToGet.Material.Color];
									//if (leafletItem.totalNumber > leafletItem.collected) {
									if (agent.getTotalLeaflet(itemToGet.Material.Color) > agent.getCollectedLeaflet(itemToGet.Material.Color)) {
										Console.WriteLine ("[" + creatureName + "] " + "Saking " + itemToGet.Material.Color);
										Console.Write (worldServer2.SendSackIt (creatureId, itemToGet.Name));
										Regex rgx = new Regex(".*" + itemToGet.Name + ".*");
										if (rgx.IsMatch(worldServer2.SendSackIt (creatureId, itemToGet.Name))){
											Console.Write(itemToGet.Name);
											agent.addCollectedLeaflet (itemToGet.Material.Color);
											jewelsDict.Remove (itemToGet.Name);
											//Consolidatedleaflets [itemToGet.Material.Color] = leafletItem;
										}
									}
								//}
							}
							break;
						}
					}
					break;
					//<<--Hirley
				default:
					break;
				}
			}
		}

		#endregion
	}
	
	
}
